@@ -1,5 +1,4 @@
 import asyncio
-import six
 import sys
 import time
 
@@ -10,24 +9,27 @@ from . import socket
 
 
 class AsyncSocket(socket.Socket):
-    def create_queue(self):
-        return asyncio.Queue()
-
     async def poll(self):
         """Wait for packets to send to the client."""
         try:
-            packets = [await asyncio.wait_for(self.queue.get(),
-                                              self.server.ping_timeout)]
+            packets = [await asyncio.wait_for(
+                self.queue.get(),
+                self.server.ping_interval + self.server.ping_timeout)]
             self.queue.task_done()
         except (asyncio.TimeoutError, asyncio.CancelledError):
             raise exceptions.QueueEmpty()
         if packets == [None]:
             return []
-        try:
-            packets.append(self.queue.get_nowait())
-            self.queue.task_done()
-        except asyncio.QueueEmpty:
-            pass
+        while True:
+            try:
+                pkt = self.queue.get_nowait()
+                self.queue.task_done()
+                if pkt is None:
+                    self.queue.put_nowait(None)
+                    break
+                packets.append(pkt)
+            except asyncio.QueueEmpty:
+                break
         return packets
 
     async def receive(self, pkt):
@@ -36,9 +38,8 @@ class AsyncSocket(socket.Socket):
                                 self.sid, packet.packet_names[pkt.packet_type],
                                 pkt.data if not isinstance(pkt.data, bytes)
                                 else '<binary>')
-        if pkt.packet_type == packet.PING:
-            self.last_ping = time.time()
-            await self.send(packet.Packet(packet.PONG, pkt.data))
+        if pkt.packet_type == packet.PONG:
+            self.schedule_ping()
         elif pkt.packet_type == packet.MESSAGE:
             await self.server._trigger_event(
                 'message', self.sid, pkt.data,
@@ -51,13 +52,11 @@ class AsyncSocket(socket.Socket):
             raise exceptions.UnknownPacketError()
 
     async def check_ping_timeout(self):
-        """Make sure the client is still sending pings.
-
-        This helps detect disconnections for long-polling clients.
-        """
+        """Make sure the client is still sending pings."""
         if self.closed:
             raise exceptions.SocketIsClosedError()
-        if time.time() - self.last_ping > self.server.ping_timeout:
+        if self.last_ping and \
+                time.time() - self.last_ping > self.server.ping_timeout:
             self.server.logger.info('%s: Client is gone, closing socket',
                                     self.sid)
             # Passing abort=False here will cause close() to write a
@@ -71,8 +70,6 @@ class AsyncSocket(socket.Socket):
         """Send a packet to the client."""
         if not await self.check_ping_timeout():
             return
-        if self.upgrading:
-            self.packet_backlog.append(pkt)
         else:
             await self.queue.put(pkt)
         self.server.logger.info('%s: Sending packet %s data %s',
@@ -90,12 +87,16 @@ class AsyncSocket(socket.Socket):
             self.server.logger.info('%s: Received request to upgrade to %s',
                                     self.sid, transport)
             return await getattr(self, '_upgrade_' + transport)(environ)
+        if self.upgrading or self.upgraded:
+            # we are upgrading to WebSocket, do not return any more packets
+            # through the polling endpoint
+            return [packet.Packet(packet.NOOP)]
         try:
             packets = await self.poll()
         except exceptions.QueueEmpty:
             exc = sys.exc_info()
             await self.close(wait=False)
-            six.reraise(*exc)
+            raise exc[1].with_traceback(exc[2])
         return packets
 
     async def handle_post_request(self, environ):
@@ -104,7 +105,7 @@ class AsyncSocket(socket.Socket):
         if length > self.server.max_http_buffer_size:
             raise exceptions.ContentTooLongError()
         else:
-            body = await environ['wsgi.input'].read(length)
+            body = (await environ['wsgi.input'].read(length)).decode('utf-8')
             p = payload.Payload(encoded_payload=body)
             for pkt in p.packets:
                 await self.receive(pkt)
@@ -120,17 +121,24 @@ class AsyncSocket(socket.Socket):
             if wait:
                 await self.queue.join()
 
+    def schedule_ping(self):
+        async def send_ping():
+            self.last_ping = None
+            await asyncio.sleep(self.server.ping_interval)
+            if not self.closing and not self.closed:
+                self.last_ping = time.time()
+                await self.send(packet.Packet(packet.PING))
+
+        self.server.start_background_task(send_ping)
+
     async def _upgrade_websocket(self, environ):
         """Upgrade the connection from polling to websocket."""
         if self.upgraded:
             raise IOError('Socket has been upgraded already')
-        if self.server._async['websocket'] is None or \
-                self.server._async['websocket_class'] is None:
+        if self.server._async['websocket'] is None:
             # the selected async mode does not support websocket
             return self.server._bad_request()
-        websocket_class = getattr(self.server._async['websocket'],
-                                  self.server._async['websocket_class'])
-        ws = websocket_class(self._websocket_handler)
+        ws = self.server._async['websocket'](self._websocket_handler)
         return await ws(environ)
 
     async def _websocket_handler(self, ws):
@@ -138,7 +146,6 @@ class AsyncSocket(socket.Socket):
         if self.connected:
             # the socket was already connected, so this is an upgrade
             self.upgrading = True  # hold packet sends during the upgrade
-            await self.queue.join()  # flush the queue first
 
             try:
                 pkt = await ws.wait()
@@ -149,15 +156,15 @@ class AsyncSocket(socket.Socket):
                     decoded_pkt.data != 'probe':
                 self.server.logger.info(
                     '%s: Failed websocket upgrade, no PING packet', self.sid)
+                self.upgrading = False
                 return
-            await ws.send(packet.Packet(
-                packet.PONG,
-                data=six.text_type('probe')).encode(always_bytes=False))
+            await ws.send(packet.Packet(packet.PONG, data='probe').encode())
             await self.queue.put(packet.Packet(packet.NOOP))  # end poll
 
             try:
                 pkt = await ws.wait()
             except IOError:  # pragma: no cover
+                self.upgrading = False
                 return
             decoded_pkt = packet.Packet(encoded_packet=pkt)
             if decoded_pkt.packet_type != packet.UPGRADE:
@@ -166,13 +173,9 @@ class AsyncSocket(socket.Socket):
                     ('%s: Failed websocket upgrade, expected UPGRADE packet, '
                      'received %s instead.'),
                     self.sid, pkt)
+                self.upgrading = False
                 return
             self.upgraded = True
-
-            # flush any packets that were sent during the upgrade
-            for pkt in self.packet_backlog:
-                await self.queue.put(pkt)
-            self.packet_backlog = []
             self.upgrading = False
         else:
             self.connected = True
@@ -191,7 +194,7 @@ class AsyncSocket(socket.Socket):
                     break
                 try:
                     for pkt in packets:
-                        await ws.send(pkt.encode(always_bytes=False))
+                        await ws.send(pkt.encode())
                 except:
                     break
         writer_task = asyncio.ensure_future(writer())
@@ -203,7 +206,9 @@ class AsyncSocket(socket.Socket):
             p = None
             wait_task = asyncio.ensure_future(ws.wait())
             try:
-                p = await asyncio.wait_for(wait_task, self.server.ping_timeout)
+                p = await asyncio.wait_for(
+                    wait_task,
+                    self.server.ping_interval + self.server.ping_timeout)
             except asyncio.CancelledError:  # pragma: no cover
                 # there is a bug (https://bugs.python.org/issue30508) in
                 # asyncio that causes a "Task exception never retrieved" error
@@ -222,8 +227,6 @@ class AsyncSocket(socket.Socket):
             if p is None:
                 # connection closed by client
                 break
-            if isinstance(p, six.text_type):  # pragma: no cover
-                p = p.encode('utf-8')
             pkt = packet.Packet(encoded_packet=p)
             try:
                 await self.receive(pkt)
