@@ -9,9 +9,8 @@ import signal
 import ssl
 import threading
 import time
+import urllib
 
-import six
-from six.moves import urllib
 try:
     import requests
 except ImportError:  # pragma: no cover
@@ -27,9 +26,6 @@ from . import payload
 default_logger = logging.getLogger('engineio.client')
 connected_clients = []
 
-if six.PY2:  # pragma: no cover
-    ConnectionError = OSError
-
 
 def signal_handler(sig, frame):
     """SIGINT handler.
@@ -85,11 +81,8 @@ class Client(object):
         global original_signal_handler
         if original_signal_handler is None and \
                 threading.current_thread() == threading.main_thread():
-            if not hasattr(signal, 'SIGBREAK'):
-                sig = signal.SIGINT
-            else:  # pragma: no cover
-                sig = signal.SIGBREAK
-            original_signal_handler = signal.signal(sig, signal_handler)
+            original_signal_handler = signal.signal(signal.SIGINT,
+                                                    signal_handler)
         self.handlers = {}
         self.base_url = None
         self.transports = None
@@ -98,13 +91,10 @@ class Client(object):
         self.upgrades = None
         self.ping_interval = None
         self.ping_timeout = None
-        self.pong_received = True
         self.http = http_session
         self.ws = None
         self.read_loop_task = None
         self.write_loop_task = None
-        self.ping_loop_task = None
-        self.ping_loop_event = None
         self.queue = None
         self.state = 'disconnected'
         self.ssl_verify = ssl_verify
@@ -115,8 +105,7 @@ class Client(object):
             self.logger = logger
         else:
             self.logger = default_logger
-            if not logging.root.handlers and \
-                    self.logger.level == logging.NOTSET:
+            if self.logger.level == logging.NOTSET:
                 if logger:
                     self.logger.setLevel(logging.INFO)
                 else:
@@ -186,7 +175,7 @@ class Client(object):
             raise ValueError('Client is not in a disconnected state')
         valid_transports = ['polling', 'websocket']
         if transports is not None:
-            if isinstance(transports, six.string_types):
+            if isinstance(transports, str):
                 transports = [transports]
             transports = [transport for transport in transports
                           if transport in valid_transports]
@@ -206,19 +195,14 @@ class Client(object):
         if self.read_loop_task:
             self.read_loop_task.join()
 
-    def send(self, data, binary=None):
+    def send(self, data):
         """Send a message to a client.
 
         :param data: The data to send to the client. Data can be of type
                      ``str``, ``bytes``, ``list`` or ``dict``. If a ``list``
                      or ``dict``, the data will be serialized as JSON.
-        :param binary: ``True`` to send packet as binary, ``False`` to send
-                       as text. If not given, unicode (Python 2) and str
-                       (Python 3) are sent as text, and str (Python 2) and
-                       bytes (Python 3) are sent as binary.
         """
-        self._send_packet(packet.Packet(packet.MESSAGE, data=data,
-                                        binary=binary))
+        self._send_packet(packet.Packet(packet.MESSAGE, data=data))
 
     def disconnect(self, abort=False):
         """Disconnect from the server.
@@ -312,10 +296,10 @@ class Client(object):
                 'Unexpected status code {} in server response'.format(
                     r.status_code), arg)
         try:
-            p = payload.Payload(encoded_payload=r.content)
+            p = payload.Payload(encoded_payload=r.content.decode('utf-8'))
         except ValueError:
-            six.raise_from(exceptions.ConnectionError(
-                'Unexpected response from server'), None)
+            raise exceptions.ConnectionError(
+                'Unexpected response from server') from None
         open_packet = p.packets[0]
         if open_packet.packet_type != packet.OPEN:
             raise exceptions.ConnectionError(
@@ -343,7 +327,6 @@ class Client(object):
                 return
 
         # start background tasks associated with this client
-        self.ping_loop_task = self.start_background_task(self._ping_loop)
         self.write_loop_task = self.start_background_task(self._write_loop)
         self.read_loop_task = self.start_background_task(
             self._read_loop_polling)
@@ -414,7 +397,6 @@ class Client(object):
                     parsed_url = urllib.parse.urlparse(
                         proxy_url if '://' in proxy_url
                         else 'scheme://' + proxy_url)
-                    print(parsed_url)
                     extra_options['http_proxy_host'] = parsed_url.hostname
                     extra_options['http_proxy_port'] = parsed_url.port
                     extra_options['http_proxy_auth'] = (
@@ -440,8 +422,7 @@ class Client(object):
             else:
                 raise exceptions.ConnectionError('Connection error')
         if upgrade:
-            p = packet.Packet(packet.PING,
-                              data=six.text_type('probe')).encode()
+            p = packet.Packet(packet.PING, data='probe').encode()
             try:
                 ws.send(p)
             except Exception as e:  # pragma: no cover
@@ -492,9 +473,9 @@ class Client(object):
             connected_clients.append(self)
             self._trigger_event('connect', run_async=False)
         self.ws = ws
+        self.ws.settimeout(self.ping_interval + self.ping_timeout)
 
         # start background tasks associated with this client
-        self.ping_loop_task = self.start_background_task(self._ping_loop)
         self.write_loop_task = self.start_background_task(self._write_loop)
         self.read_loop_task = self.start_background_task(
             self._read_loop_websocket)
@@ -509,8 +490,8 @@ class Client(object):
             pkt.data if not isinstance(pkt.data, bytes) else '<binary>')
         if pkt.packet_type == packet.MESSAGE:
             self._trigger_event('message', pkt.data, run_async=True)
-        elif pkt.packet_type == packet.PONG:
-            self.pong_received = True
+        elif pkt.packet_type == packet.PING:
+            self._send_packet(packet.Packet(packet.PONG, pkt.data))
         elif pkt.packet_type == packet.CLOSE:
             self.disconnect(abort=True)
         elif pkt.packet_type == packet.NOOP:
@@ -568,7 +549,7 @@ class Client(object):
             scheme += 's'
 
         return ('{scheme}://{netloc}/{path}/?{query}'
-                '{sep}transport={transport}&EIO=3').format(
+                '{sep}transport={transport}&EIO=4').format(
                     scheme=scheme, netloc=parsed_url.netloc,
                     path=engineio_path, query=parsed_url.query,
                     sep='&' if parsed_url.query else '',
@@ -578,28 +559,6 @@ class Client(object):
         """Generate the Engine.IO query string timestamp."""
         return '&t=' + str(time.time())
 
-    def _ping_loop(self):
-        """This background task sends a PING to the server at the requested
-        interval.
-        """
-        self.pong_received = True
-        if self.ping_loop_event is None:
-            self.ping_loop_event = self.create_event()
-        else:
-            self.ping_loop_event.clear()
-        while self.state == 'connected':
-            if not self.pong_received:
-                self.logger.info(
-                    'PONG response has not been received, aborting')
-                if self.ws:
-                    self.ws.close(timeout=0)
-                self.queue.put(None)
-                break
-            self.pong_received = False
-            self._send_packet(packet.Packet(packet.PING))
-            self.ping_loop_event.wait(timeout=self.ping_interval)
-        self.logger.info('Exiting ping task')
-
     def _read_loop_polling(self):
         """Read packets by polling the Engine.IO server."""
         while self.state == 'connected':
@@ -619,7 +578,7 @@ class Client(object):
                 self.queue.put(None)
                 break
             try:
-                p = payload.Payload(encoded_payload=r.content)
+                p = payload.Payload(encoded_payload=r.content.decode('utf-8'))
             except ValueError:
                 self.logger.warning(
                     'Unexpected packet from server, aborting')
@@ -630,10 +589,6 @@ class Client(object):
 
         self.logger.info('Waiting for write loop task to end')
         self.write_loop_task.join()
-        self.logger.info('Waiting for ping loop task to end')
-        if self.ping_loop_event:  # pragma: no cover
-            self.ping_loop_event.set()
-        self.ping_loop_task.join()
         if self.state == 'connected':
             self._trigger_event('disconnect', run_async=False)
             try:
@@ -649,6 +604,11 @@ class Client(object):
             p = None
             try:
                 p = self.ws.recv()
+            except websocket.WebSocketTimeoutException:
+                self.logger.warning(
+                    'Server has stopped communicating, aborting')
+                self.queue.put(None)
+                break
             except websocket.WebSocketConnectionClosedException:
                 self.logger.warning(
                     'WebSocket connection was closed, aborting')
@@ -660,8 +620,6 @@ class Client(object):
                     str(e))
                 self.queue.put(None)
                 break
-            if isinstance(p, six.text_type):  # pragma: no cover
-                p = p.encode('utf-8')
             try:
                 pkt = packet.Packet(encoded_packet=p)
             except Exception as e:  # pragma: no cover
@@ -673,10 +631,6 @@ class Client(object):
 
         self.logger.info('Waiting for write loop task to end')
         self.write_loop_task.join()
-        self.logger.info('Waiting for ping loop task to end')
-        if self.ping_loop_event:  # pragma: no cover
-            self.ping_loop_event.set()
-        self.ping_loop_task.join()
         if self.state == 'connected':
             self._trigger_event('disconnect', run_async=False)
             try:
@@ -738,7 +692,7 @@ class Client(object):
                 # websocket
                 try:
                     for pkt in packets:
-                        encoded_packet = pkt.encode(always_bytes=False)
+                        encoded_packet = pkt.encode()
                         if pkt.binary:
                             self.ws.send_binary(encoded_packet)
                         else:
